#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>

#define crc32		crc32_be
#define crc32init	crc32init_be

unsigned char data[4096*4096];

unsigned long crc32_be(unsigned long crc, unsigned char const *p, size_t len);
unsigned long crc32_le(unsigned long crc, unsigned char const *p, size_t len);
void crc32init_be(void);
void crc32init_le(void);

void fatal_error(const char *why)
{
  perror(why);
  exit(1);
}


int checksum(FILE *fd, unsigned char *data, unsigned long len, const char *desc)
{
  unsigned long crc = 0, scrc;

  fread(data, sizeof(long), len, fd);
  crc = crc32(crc, data, sizeof(long) * len);
  crc ^= 0xFFFFFFFF;
  fread(&scrc, sizeof(long), 1, fd);

  if (crc != scrc) {
    fseek(fd, -4, SEEK_CUR);
    fwrite(&crc, sizeof(crc), 1, fd);
	fseek(fd, 0, SEEK_CUR);		// For Windows.
    printf("%10s: calc %#010lx / file %#010lx: CRC fixed.\n", desc, crc, scrc);
    return 1;
  }
  return 0;
}


int main(int argc, char *argv[])
{
  FILE *fd;
  unsigned long table_count, field_count, row_count, row_size, row_used, idx_count;
  unsigned long empty, i, changed = 0;
  char table_name[64];

  if (argc < 2) {
    printf("%s: need name of Madden franchise to adjust CRCs on.\n", argv[0]);
    exit(1);
  }

  if (!(fd = fopen(argv[1], "r+b")))
    fatal_error(argv[1]);

  crc32init();
  changed += checksum(fd, data, 5, "Header");
  table_count = data[16];
  changed += checksum(fd, data, table_count * 2, "Index");

  for (i = 0; i < table_count; i++) {
    sprintf(table_name, "%3lu: Header", i);
    changed += checksum(fd, data, 8, table_name);

    row_size  = (data[ 5] << 8) + data[ 4];
    row_count = (data[17] << 8) + data[16];
    row_used  = (data[19] << 8) + data[18];
    idx_count = (data[26] << 8) + data[25];
    empty = (data[0] & 0x10);
    field_count = data[24];
//    printf("rs=%lu rc=%lu ru=%lu fc=%lu ic=%lu e=%lu s=%lu\n", row_size, row_count, row_used, field_count, idx_count, empty, row_size * row_count);

    sprintf(table_name, "%3lu: Data  ", i);
    changed += checksum(fd, data, field_count * 4 + (!empty ? row_size * row_count / sizeof(long) : 0) + idx_count * 4, table_name);
  }

  printf("Done. %lu changes.\n", changed);
  fclose(fd);
  return 0;
}
